/* CHANGES from Sandwich2_v6:
- added logic for HCAL self-triggering 

 VERSION Sandwich2_v6 is simplified and tailored to
   the HTR production test.

CHANGES from Sandwich2_v3:
- fixed ucf: #NET   "PH_RxD<1>"   LOC = "C21"; and NET   "PH_TxD<0>" 
- on ucf, periodo constraint on SCLK
- fixed DCM synthesis attributes
*/  

`include "define.v"
module SandwichTop 
 ( 
 input  CLKp, CLKn, // 40MHz differential RX_CLK
 input  BCOp, BCOn, // RX_BC0 common to all SLB boards
 input  SLB_ID,
 input  TTC_CK40Des1, TTC_BRCST_STRB1, TTC_BCR,
 input  SCLK,       // 80 MHz Clock from onboard crystal
 input  LOC_nCS,  LocWRITE_b, 
 input  [7:2] TTC_BRCST,      
 input  [8:0] SYNC1, SYNC2, SYNC3, SYNC4, 
              SYNC5, SYNC6, SYNC7, SYNC8,   // Data In
 input  [`LB_Addr_Size-1:0]  LocAdd,  // The Address bus.
 inout  [`REG_WIDTH-1:0]     LocBus,  // The bidir Data bus 
 output SLB_CFGDONE,
 output LVDS1n, LVDS1p, LVDS2n, LVDS2p,// to MDR14 J10
 output REFCLK, TRSTN,  LED_CONTROL          );
 
 wire  VME_Hard_rst, VME_Soft_rst, VME_Start, VME_Stop, VME_LVDSout,
       run_status, DisSODV, BunchEr, GateTTCcommands, Hard_rst, Soft_rst;
 wire [15:0] TTC_test0, TTC_test1, TTC_test2, TTC_test3,
             TTC_test4, TTC_test5, TTC_test6, TTC_test7,
             TTC_test8, TTC_test9, TTC_testA, TTC_testB,
             TTC_testC, TTC_testD, TTC_testE, TTC_testF;
 wire [`BCN_WIDTH-1:0] MaxBCN; 
 wire [8:1]  TPSpyFifoEmpty, TPSpyFifoFull, TPSpyFifoRd,
             InstantCntErr,  InstantR1sErr;
 wire [15:0] AccCntErr, AccR1sErr;  
 wire [8:0]  TPSpyFIFO1_out, TPSpyFIFO2_out, TPSpyFIFO3_out, TPSpyFIFO4_out,
             TPSpyFIFO5_out, TPSpyFIFO6_out, TPSpyFIFO7_out, TPSpyFIFO8_out;
 wire [6:0]  pad7_1,  pad7_2,  pad7_3,  pad7_4;
 wire [26:0] pad27_1, pad27_2, pad27_3, pad27_4,
             pad27_5, pad27_6, pad27_7, pad27_8;
 reg VME_LVDSout_D, LVDSout;
 reg [7:0] LastTTCcommand;
 reg [8:0] SYNC1D, SYNC2D, SYNC3D, SYNC4D, SYNC5D, SYNC6D, SYNC7D, SYNC8D;
 reg [14:0] Or;
 reg [15:0] RxClkCnt, RxBC0Cnt, TClkCnt;

//  ********************************************************************
//      ********************  CLOCK SECTION    *****************
//            *********************************************

 wire   RxClk_Buf, SCLK_buf;
 wire   Tclk0, TTC_CK, Tclk2x_dll, TTC_CK2x, TTC_DLL_lock;
 reg    [2:0] UnlockCnt; // counter for proper reset of DLL 

 // Instantiate the Differential Global Input Buffer 
  IBUFGDS_LVPECL_25 difclkbuf (.I(CLKp), .IB(CLKn), .O(RxClk_Buf));
  IBUFG sclkbuf ( .I(SCLK), .O(SCLK_buf) );
                  
// Generate a DLL reset (3-ticks) after FPGA is configured, see:
// www.xilinx.com/publications/xcellonline/xcell_51/xc_particle51.htm
  FDS flop1 (.D(1'b0), .C(SCLK_buf), .Q(out1), .S(1'b0)); 
  FD  flop2 (.D(out1), .C(SCLK_buf), .Q(out2)); 
  FD  flop3 (.D(out2), .C(SCLK_buf), .Q(out3)); 
  FD  flop4 (.D(out3), .C(SCLK_buf), .Q(out4)); 
  assign ConfigRst = (out2 | out3 | out4); 

  DCM TTC_DCM
   (.CLKIN(TTC_CK40Des1), .CLKFB(TTC_CK),  .RST(ConfigRst),  
    .CLK0(Tclk0),   .CLK90(),  .CLK180(),  .CLK270(),
    .CLK2X(),  .CLK2X180(Tclk2x_dll),  // here u can use the inverted clock.
    .CLKDV(),   .CLKFX(),   .CLKFX180(),
    .LOCKED(TTC_DLL_lock),  .STATUS(),
    .DSSEN(1'b0),
    .PSCLK(1'b0),  .PSEN(1'b0),  .PSINCDEC(1'b0), .PSDONE() );

  BUFG TCK40BUF (.I(Tclk0), .O(TTC_CK));  
  BUFG TCK2XBUF (.I(Tclk2x_dll), .O(TTC_CK2x)); 

 assign REFCLK = Tclk2x_dll; // to U2A and then to Wisc_Mezzanine.

//       *************  END OF CLOCK SECTION  *****************
//  *****************************************************************

LED_Sine breathing(.clock(SCLK_buf), .nReset(1'b1), .led(LED_CONTROL));

 assign SLB_CFGDONE = SLB_ID; // in case ECAL TCC uses this board
 assign TRSTN = 1'b1; // Wisc. Rec_Mezz JTAG reset.
 IBUFDS_LVPECL_25 RxBC0buf (.I(BCOp), .IB(BCOn), .O(RxBC0));
 OBUFDS_LVDS_25 pair1 (.I(LVDSout), .O(LVDS1p), .OB(LVDS1n));
 OBUFDS_LVDS_25 pair2 (.I(LVDSout), .O(LVDS2p), .OB(LVDS2n));

// **** Register, Spy and Check the HTR data ***

 always @ (posedge SCLK_buf) begin 
    //register data as they go to 2 destinations.
  SYNC1D <= SYNC1;  SYNC2D <= SYNC2;  SYNC3D <= SYNC3;  SYNC4D <= SYNC4;  
  SYNC5D <= SYNC5;  SYNC6D <= SYNC6;  SYNC7D <= SYNC7;  SYNC8D <= SYNC8;  
    // "OR" of 24 muon bits
  Or[0]  <= | SYNC1D[4:0];
  Or[1]  <= | SYNC1D[8:5];
  Or[2]  <= | SYNC2D[4:0];
  Or[3]  <= | SYNC2D[8:5];
  Or[4]  <= | SYNC3D[5:0];
  Or[5]  <= | SYNC5D[4:0];
  Or[6]  <= | SYNC5D[8:5];
  Or[7]  <= | SYNC6D[4:0];
  Or[8]  <= | SYNC6D[8:5];
  Or[9]  <= | SYNC7D[5:0];
  Or[10] <= | Or[4:0];
  Or[11] <= | Or[9:5];
  Or[12] <= | Or[11:10];
  Or[13] <= VME_LVDSout | Or[12];     // stretch here
  Or[14] <= Or[13];
  LVDSout <= VME_LVDSout | Or[12] | Or[13] | Or[14];
 end

fifo_ic_v2  TPSpyFIFO1 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({27'b0, SYNC1D}), 
   .write_clock_in(SCLK_buf),              .write_enable_in(run_status),
   .read_clock_in(SCLK_buf),               .read_enable_in(TPSpyFifoRd[1]),
   .read_data_out({pad27_1, TPSpyFIFO1_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[1]),            .empty_out(TPSpyFifoEmpty[1])  );

fifo_ic_v2  TPSpyFIFO2 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({27'b0, SYNC2D}), 
   .write_clock_in(SCLK_buf),              .write_enable_in(run_status),
   .read_clock_in(SCLK_buf),               .read_enable_in(TPSpyFifoRd[2]),
   .read_data_out({pad27_2, TPSpyFIFO2_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[2]),            .empty_out(TPSpyFifoEmpty[2])  );

fifo_ic_v2  TPSpyFIFO3 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({27'b0, SYNC3D}), 
   .write_clock_in(SCLK_buf),              .write_enable_in(run_status),
   .read_clock_in(SCLK_buf),               .read_enable_in(TPSpyFifoRd[3]),
   .read_data_out({pad27_3, TPSpyFIFO3_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[3]),            .empty_out(TPSpyFifoEmpty[3])  );

fifo_ic_v2  TPSpyFIFO4 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({27'b0, SYNC4D}), 
   .write_clock_in(SCLK_buf),              .write_enable_in(run_status),
   .read_clock_in(SCLK_buf),               .read_enable_in(TPSpyFifoRd[4]),
   .read_data_out({pad27_4, TPSpyFIFO4_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[4]),            .empty_out(TPSpyFifoEmpty[4])  );

fifo_ic_v2  TPSpyFIFO5 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({27'b0, SYNC5D}), 
   .write_clock_in(SCLK_buf),              .write_enable_in(run_status),
   .read_clock_in(SCLK_buf),               .read_enable_in(TPSpyFifoRd[5]),
   .read_data_out({pad27_5, TPSpyFIFO5_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[5]),            .empty_out(TPSpyFifoEmpty[5])  );

fifo_ic_v2  TPSpyFIFO6 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({27'b0, SYNC6D}), 
   .write_clock_in(SCLK_buf),              .write_enable_in(run_status),
   .read_clock_in(SCLK_buf),               .read_enable_in(TPSpyFifoRd[6]),
   .read_data_out({pad27_6, TPSpyFIFO6_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[6]),            .empty_out(TPSpyFifoEmpty[6])  );

fifo_ic_v2  TPSpyFIFO7 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({27'b0, SYNC7D}), 
   .write_clock_in(SCLK_buf),              .write_enable_in(run_status),
   .read_clock_in(SCLK_buf),               .read_enable_in(TPSpyFifoRd[7]),
   .read_data_out({pad27_7, TPSpyFIFO7_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[7]),            .empty_out(TPSpyFifoEmpty[7])  );

fifo_ic_v2  TPSpyFIFO8 
 ( .fifo_gsr_in(VME_Hard_rst),             .write_data_in({27'b0, SYNC8D}), 
   .write_clock_in(SCLK_buf),              .write_enable_in(run_status),
   .read_clock_in(SCLK_buf),               .read_enable_in(TPSpyFifoRd[8]),
   .read_data_out({pad27_8, TPSpyFIFO8_out}), .fifostatus_out(),
   .full_out(TPSpyFifoFull[8]),             .empty_out(TPSpyFifoEmpty[8])  );

ErrorChecker ErrorChecker (
    .TTC_CK2x(TTC_CK2x),   .reset(Soft_rst), 
    .SYNC1(SYNC1D),        .SYNC2(SYNC2D), 
    .SYNC3(SYNC3D),        .SYNC4(SYNC4D), 
    .SYNC5(SYNC5D),        .SYNC6(SYNC6D), 
    .SYNC7(SYNC7D),        .SYNC8(SYNC8D), 
    .InstantCntErr(InstantCntErr),  .InstantR1sErr(InstantR1sErr), 
    .AccCntErr(AccCntErr),          .AccR1sErr(AccR1sErr)    );

////////////////////////////////////////////////////////

 always @ (posedge TTC_CK40Des1)        
    if (TTC_BRCST_STRB1)  LastTTCcommand <= {TTC_BRCST, 1'b0, TTC_BCR};

LocBusSlave LocBusSlave (
    .clk(SCLK_buf),      .rst(1'b0), 
    .LocAddr(LocAdd),    .LocData(LocBus), 
    .LocCS_b(LOC_nCS),   .LocWRITE_b(LocWRITE_b), 
    .LB_STATE(),   .Din(), .Dout(),      .Tri(), 
    .RxDLL_lock(1'b0), .TTC_DLL_lock(TTC_DLL_lock), .UnlockCnt(UnlockCnt),
    .DLL_status(16'b0),
    .ErrA(1'b0), .IdleA(1'b0), .KChA(1'b0), 
    .ErrB(1'b0), .IdleB(1'b0), .KChB(1'b0),
    .ErrC(1'b0), .IdleC(1'b0), .KChC(1'b0),
    .ErrD(1'b0), .IdleD(1'b0), .KChD(1'b0),
    .LastTTCcommand(LastTTCcommand), .GateTTCcommands(GateTTCcommands), 
    .VME_Hard_rst(VME_Hard_rst),     .VME_Soft_rst(VME_Soft_rst), 
    .VME_Start(VME_Start),           .VME_Stop(VME_Stop), 
    .VME_VitResetN(),       .VME_LVDSout(VME_LVDSout), 
    .run_status(run_status),         .DisSODV(DisSODV),
    .InstantCntErr(InstantCntErr),   .InstantR1sErr(InstantR1sErr), 
    .AccCntErr(AccCntErr),           .AccR1sErr(AccR1sErr),
    .RxClkCnt(RxClkCnt), .RxBC0Cnt(RxBC0Cnt),
    .TClkCnt(TClkCnt), .TxBC0Cnt(16'b0),
    .TPSpyFifoRd(TPSpyFifoRd),
    .TPSpyFifoEmpty(TPSpyFifoEmpty),   .TPSpyFifoFull(TPSpyFifoFull), 
    .TPSpyFIFO1_out(TPSpyFIFO1_out),   .TPSpyFIFO2_out(TPSpyFIFO2_out),
    .TPSpyFIFO3_out(TPSpyFIFO3_out),   .TPSpyFIFO4_out(TPSpyFIFO4_out),
    .TPSpyFIFO5_out(TPSpyFIFO5_out),   .TPSpyFIFO6_out(TPSpyFIFO6_out),
    .TPSpyFIFO7_out(TPSpyFIFO7_out),   .TPSpyFIFO8_out(TPSpyFIFO8_out),
    .VitSpyFifoEmpty(4'b0), .VitSpyFifoFull(4'b0),
    .VitSpyFifoRd(),      
    .VitSpyFIFO1_out(11'b0), .VitSpyFIFO2_out(11'b0),
    .VitSpyFIFO3_out(11'b0), .VitSpyFIFO4_out(11'b0),
    .TTC_test0(TTC_test0),       .TTC_test1(TTC_test1), 
    .TTC_test2(TTC_test2),       .TTC_test3(TTC_test3), 
    .TTC_test4(TTC_test4),       .TTC_test5(TTC_test5), 
    .TTC_test6(TTC_test6),       .TTC_test7(TTC_test7), 
    .TTC_test8(TTC_test8),       .TTC_test9(TTC_test9), 
    .TTC_testA(TTC_testA),       .TTC_testB(TTC_testB), 
    .TTC_testC(TTC_testC),       .TTC_testD(TTC_testD), 
    .TTC_testE(TTC_testE),       .TTC_testF(TTC_testF), 
    .MaxBCN(MaxBCN),             .BunchEr(BunchEr), 
    .CounterModeSel()    );

 commander commander (  
  .clk(SCLK_buf),    .GateTTCcommands(GateTTCcommands),
  .TTC_Hard_rst(1'b0), .TTC_Soft_rst(1'b0),
  .TTC_Start(1'b0), .TTC_Stop(1'b0), 
  .VME_Hard_rst(VME_Hard_rst), .VME_Soft_rst(VME_Soft_rst), 
  .VME_Start(VME_Start), .VME_Stop(VME_Stop), 
  .Hard_rst(Hard_rst),  .Soft_rst(Soft_rst), 
  .run_status(run_status)       );

 always @ (posedge RxClk_Buf) begin 
   if (Soft_rst) begin
      RxClkCnt <= 0;  RxBC0Cnt <= 0;
   end
   else if (run_status) begin
      RxClkCnt <= RxClkCnt + 1;
      if (RxBC0) RxBC0Cnt <= RxBC0Cnt + 1;
   end
 end // always

 always @ (posedge TTC_CK) begin 
   if (Soft_rst)         TClkCnt <= 0; 
   else if (run_status)  TClkCnt <= TClkCnt + 1;
 end // always

endmodule